program mp_wrapper;

uses
  windows, sysutils,
  hge, hgesprite, hgefont,
  magic, mp, mp_wrap, win_data;
  
var
  hge : IHGE;
  system_font : IHGEfont;
  time, last_time : cardinal;
  mp : TMP_Manager;
  cur : integer = 0;
  particles_render : integer;
  
procedure init;
var
  pos : TMP_POSITION;
  i : integer;
  fname, texture_folder : AnsiString;
begin
  mp_wrap.hge := HGE;
  last_time := 0;
  particles_render := 0;
  system_font := THGEFont.Create('system.fnt');

  mp := Tmp_manager.Create;
  mp.Init(MAGIC_INTERPOLATION_ENABLE, MAGIC_NOLOOP, MAGIC_CHANGE_EMITTER_ONLY, GetPathToTemp);

  //      
  fname := GetFirstFile;
  while fname <> '' do begin
    mp.LoadEmittersFromFile(fname);
    fname := GetNextFile;
  end;

  texture_folder := GetPathToTexture;
  mp.LoadTextures(texture_folder,1024,1024,1,0.1);

  mp.CloseFiles;

  mp.Stop;

  mp.GetEmitter(cur).SetState(MAGIC_STATE_UPDATE);

  for i := 0 to mp.GetEmitterCount - 1 do begin
    mp.GetEmitter(i).GetPosition(pos);
    pos.x := pos.x + hge.System_GetState(HGE_SCREENWIDTH) div 2;
    pos.y := pos.y + hge.System_GetState(HGE_SCREENHEIGHT) div 2;
    mp.GetEmitter(i).Move(pos);
  end;
end;

function frame_func : boolean;
var
  rate : double;
  emitter : TMP_Emitter;
begin
  result := HGE.Input_KeyUp(HGEK_ESCAPE);
  if result then exit;

  rate := 0.01;
  time := GetTick;
  if time > last_time then
  begin
    rate := time - last_time;
    last_time := time;
    if rate > 500 then rate := 0.01;
  end;

  mp.Update(rate);

  emitter := mp.GetEmitter(cur);
  if emitter.GetState = MAGIC_STATE_STOP then
  begin
    inc(cur);
    if cur >= mp.GetEmitterCount then
      cur := 0;
    emitter := mp.GetEmitter(cur);
    emitter.SetState(MAGIC_STATE_UPDATE);
  end;
end;

function render_func : boolean;
var
  i, k : integer;
  em : TMP_Emitter;
begin
  hge.Gfx_BeginScene;
  hge.Gfx_Clear($0);

  particles_render := 0;
  k := mp.GetEmitterCount;
  for i := 0 to k - 1 do begin
    em := mp.GetEmitter(i);
    particles_render := particles_render + em.Render;
  end;
    
  system_font.SetColor($FF00FF00);
  system_font.Render(10,10,0,'fps='+inttostr(hge.Timer_GetFPS));
  system_font.Render(10,30,0,'particles count='+inttostr(particles_render));
  system_font.Render(10,50,0,'current emitter='+inttostr(cur+1));
  system_font.Render(10,70,0,'emitters='+inttostr(mp.GetEmitterCount));
  result := false;

  hge.Gfx_EndScene;
end;


begin
  ReportMemoryLeaksOnShutdown := true;
  hge := HGECreate(HGE_VERSION);
  HGE.System_SetState(HGE_TITLE, 'Magic Particles (www.astralax.com) Delphi wrapper for HGE');
  HGE.System_SetState(HGE_LOGFILE, 'log.txt');
  HGE.System_SetState(HGE_WINDOWED, true);
  //    
  HGE.System_SetState(HGE_FRAMEFUNC,frame_func);
  //    
  HGE.System_SetState(HGE_RENDERFUNC,render_func);
  HGE.System_SetState(HGE_FPS,HGEFPS_UNLIMITED);
  HGE.System_SetState(HGE_TEXTUREFILTER,false);

  if HGE.System_Initiate then begin
    init;
    HGE.System_Start;
  end;

  mp.Free;
  HGE.System_Shutdown;
  system_font := nil;
  HGE := nil;
end.
